/*******************************************************************************
* NXP Semiconductors
* (c) Copyright 2017 NXP Semiconductors
* ALL RIGHTS RESERVED.
********************************************************************************
Services performed by NXP in this matter are performed AS IS and without any
warranty. CUSTOMER retains the final decision relative to the total design
and functionality of the end product. NXP neither guarantees nor will be held
liable by CUSTOMER for the success of this project.
NXP DISCLAIMS ALL WARRANTIES, EXPRESSED, IMPLIED OR STATUTORY INCLUDING,
BUT NOT LIMITED TO, IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR
A PARTICULAR PURPOSE ON ANY HARDWARE, SOFTWARE ORE ADVISE SUPPLIED
TO THE PROJECT BY NXP, AND OR NAY PRODUCT RESULTING FROM NXP SERVICES.
IN NO EVENT SHALL NXP BE LIABLE FOR INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING
OUT OF THIS AGREEMENT.
CUSTOMER agrees to hold NXP harmless against any and all claims demands
or actions by anyone on account of any damage, or injury, whether commercial,
contractual, or tortuous, rising directly or indirectly as a result
of the advise or assistance supplied CUSTOMER in connection with product,
services or goods supplied under this Agreement.
********************************************************************************
* File:             main.c
* Owner:            Daniel Martynek
* Version:          1.0
* Date:             Apr-20-2017
* Classification:   General Business Information
* Brief:            Verify backdoor key access
********************************************************************************
********************************************************************************
* Detailed Description:
* This example shows backdoor key access.
* The MCU is secured in the start-up code (startup_S32K144.S),
* see Flash Configuration, backdoor comparison key: 0xAAAAAAAAAAAAAAAA
* After download, the MCU is secure and the debugger is disconnected.
* Power-on reset is needed to run the MCU stand alone.
* Red LED on the EVB indicates the MCU is secure.
* Verify backdoor key command, which will unlock the MCU and which is executed from RAM,
* is delayed with dummy delay loop.
* After that the MCU will be unsecured and the LED will turn BLUE.
* * ------------------------------------------------------------------------------
* Test HW:         S32144EVB-Q100
* MCU:             S32K144 0N47T
* Debugger:        S32DS 1.3
* Target:          internal_FLASH
*
********************************************************************************
Revision History:
1.0     Apr-20-2017     Daniel Martynek    Initial Version
*******************************************************************************/
/*******************************************************************************
* Includes
*******************************************************************************/
#include "S32K144.h" /* include peripheral declarations S32K144 */
#include <string.h>
/*******************************************************************************
* External objects
*******************************************************************************/
extern unsigned long __MY_RAM_ADR;
extern unsigned long __MY_RAM_ROM_ADR;
extern unsigned long __MY_RAM_SIZE;
/*******************************************************************************
* Global variables
*******************************************************************************/
unsigned int i = 0;

/*******************************************************************************
* Local functions
*******************************************************************************/
__attribute__ ((section(".MyRamCode")))             		 // place the function below into .MyRamCode section
void backdoorKeyVerify(void) __attribute__ ((longcall));  	 // declare the function as "far"
__attribute__ ((section(".text")))
void portInit(void);
void led(void);

/*******************************************************************************
Function Name : backdoorKeyVerify
Date          : Apr-20-2017
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : Verify backdoor key access
Issues        : The flash command must must be executed from RAM
*******************************************************************************/
void backdoorKeyVerify(void){
	while((FTFC->FSTAT & FTFC_FSTAT_CCIF_MASK) == 0);	//wait if operation in progress
	FTFC->FSTAT = FTFC_FSTAT_ACCERR_MASK | FTFC_FSTAT_FPVIOL_MASK;

	FTFC->FCCOB[3] = 0x45;	// Backdoor Key Verify command (0x45)

	FTFC->FCCOB[7] =  0xAA;	// key
	FTFC->FCCOB[6] =  0xAA;
	FTFC->FCCOB[5] =  0xAA;
	FTFC->FCCOB[4] =  0xAA;
	FTFC->FCCOB[11] = 0xAA;
	FTFC->FCCOB[10] = 0xAA;
	FTFC->FCCOB[9] =  0xAA;
	FTFC->FCCOB[8] =  0xAA;

	FTFC->FSTAT = FTFC_FSTAT_CCIF_MASK;	//launch command
	while((FTFC->FSTAT & FTFC_FSTAT_CCIF_MASK) == 0);	//wait for done
}
/*******************************************************************************
Function Name : portInit
Date          : Apr-20-2017
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : Port configuration
Issues        : NONE
*******************************************************************************/
void portInit(void)
{
	PCC-> PCCn[PCC_PORTD_INDEX] = PCC_PCCn_CGC_MASK;   // Enable clock to PortD - BUS_CLK
	PTD->PDDR |= (1 << 15);          // Port D15: Data Direction  (output)
	PORTD->PCR[15] = 0x00000100;     // Port D15: MUX = GPIO
	PTD-> PSOR |= (1 << 15);         // Clear Output on port D15 (red LED off)

	PTD->PDDR |= (1 << 0);          // Port D15: Data Direction  (output)
	PORTD->PCR[0] = 0x00000100;     // Port D15: MUX = GPIO
	PTD-> PSOR |= (1 << 0);         // Clear Output on port D15 (blue LED off)
}
/*******************************************************************************
Function Name : led
Date          : Apr-20-2017
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : LEDs show whether the MCU is secure
Issues        : NONE
*******************************************************************************/
void led(void){
	if (FTFC->FSEC & (1 << 0))        // SEC = 0b11 (MCU security status is secure)
	{
	  	  PTD-> PCOR |= (1 << 15);         // red LED on
	  	  PTD-> PSOR |= (1 << 0);          // blue LED off
	}
	if ((FTFC->FSEC & (1 << 0)) == 0) // SEC = 0b10 (MCU security status is unsecured)
	{
		  PTD-> PSOR |= (1 << 15);         // red LED off
		  PTD-> PCOR |= (1 << 0);          // blue LED on
	}
}
/*******************************************************************************
* Global functions
*******************************************************************************/
int main(void)
{
  // Copy backdoorKeyVerify() into RAM
  memcpy(&__MY_RAM_ADR , &__MY_RAM_ROM_ADR, &__MY_RAM_SIZE);  // copy the function from flash to RAM

  // Set up LEDs
  portInit();

  // Show states of SEC bits, RED - secure, BLUE - unsecured
  led();

  // Delay loop
  for(i = 0; i < 5000000; ++i);

  // Verify backdoor key
  backdoorKeyVerify();

  // Show states of SEC bits, RED - secure, BLUE - unsecured
  led();

  while(1);

}
